; PATCH file for keyboard LEDs
; To be applied to M.A.M.E. v0.28
; 97-09-19 Sean Gugler
;
; Modifies the following files:
;   src/osdepend.h
;   src/msdos/msdos.c 
;   src/inptport.c 
;   src/inptport.c 
;   src/drivers/ccastles.c 
;   src/drivers/centiped.c 
;   src/drivers/galaxian.c 
;   src/drivers/pacman.c 
;   src/drivers/rallyx.c 
;   src/drivers/sbasketb.c 
;   src/machine/ccastles.c 
;   src/machine/centiped.c 
;   src/machine/missile.c 
;   src/machine/starwars.c 
;
;--------------------------------------------------------------------------
;
diff -rC2 new/src/osdepend.h src/src/osdepend.h
*** new/src/osdepend.h	Tue Sep  2 21:50:14 1997
--- src/src/osdepend.h	Thu Sep 18 17:39:16 1997
***************
*** 150,153 ****
--- 150,155 ----
  void osd_trak_center_y(void);
  
+ void osd_led_w(int led, int on);
+ 
  void open_page(int *x_res, int *y_res, int step);
  void close_page(void);
diff -rC2 new/src/msdos/msdos.c src/src/msdos/msdos.c
*** new/src/msdos/msdos.c	Tue Sep  2 23:13:50 1997
--- src/src/msdos/msdos.c	Fri Sep 19 15:59:44 1997
***************
*** 297,300 ****
--- 297,301 ----
  	allegro_init();
  	install_keyboard();		/* Allegro keyboard handler */
+ 	set_leds (0);
  
  	if (use_joystick)
***************
*** 356,359 ****
--- 357,361 ----
  		ACloseAudio();
  	}
+ 	set_leds (-1);
  }
  
***************
*** 1476,1479 ****
--- 1478,1500 ----
  }
  
+ static int leds=0;
+ static const int led_flags[3] = {
+   KB_NUMLOCK_FLAG, 
+   KB_CAPSLOCK_FLAG,
+   KB_SCROLOCK_FLAG
+ };
+ void osd_led_w(int led,int on) {
+   int temp=leds;
+   if (led<3) {
+     if (on&1)
+         temp |=  led_flags[led];
+     else
+         temp &= ~led_flags[led];
+     if (temp!=leds) {
+         leds=temp;
+         set_leds (leds);
+     }
+   }
+ }
  
  #define MAXPIXELS 100000
diff -rC2 new/src/inptport.h src/src/inptport.h
*** new/src/inptport.h	Sat Sep  6 16:27:48 1997
--- src/src/inptport.h	Thu Sep 18 17:34:04 1997
***************
*** 27,29 ****
--- 27,34 ----
  int input_trak_3_r(int offset);
  
+ void led_w(int led,int on);
+ void led_0_w(int offset,int data);
+ void led_1_w(int offset,int data);
+ void led_2_w(int offset,int data);
+ 
  #endif
diff -rC2 new/src/inptport.c src/src/inptport.c
*** new/src/inptport.c	Sat Sep  6 23:07:02 1997
--- src/src/inptport.c	Fri Sep 19 15:27:50 1997
***************
*** 850,851 ****
--- 850,868 ----
  	return readinputport(7);
  }
+ 
+ inline void led_w(int led,int on)
+ {
+     osd_led_w(led,on);
+ }
+ void led_0_w(int offset,int data)
+ {
+     led_w(0,data);
+ }
+ void led_1_w(int offset,int data)
+ {
+     led_w(1,data);
+ }
+ void led_2_w(int offset,int data)
+ {
+     led_w(2,data);
+ }
diff -rC2 new/src/drivers/ccastles.c src/src/drivers/ccastles.c
*** new/src/drivers/ccastles.c	Tue Sep  2 22:27:18 1997
--- src/src/drivers/ccastles.c	Fri Sep 19 14:50:02 1997
***************
*** 111,114 ****
--- 111,115 ----
  void ccastles_bitmapram_w(int offset,int data);
  void ccastles_bankswitch_w(int offset,int data);
+ void ccastles_led_0_w(int offset, int data);
  
  static struct MemoryReadAddress readmem[] =
***************
*** 139,143 ****
  	{ 0x9D80, 0x9D80, MWA_NOP },
  	{ 0x9E00, 0x9E00, MWA_NOP },
! 	{ 0x9E80, 0x9E81, MWA_NOP },
  	{ 0x9E85, 0x9E86, MWA_NOP },
  	{ 0x9E87, 0x9E87, ccastles_bankswitch_w },
--- 140,145 ----
  	{ 0x9D80, 0x9D80, MWA_NOP },
  	{ 0x9E00, 0x9E00, MWA_NOP },
! 	{ 0x9E80, 0x9E80, ccastles_led_0_w },
! 	{ 0x9E81, 0x9E81, MWA_NOP },
  	{ 0x9E85, 0x9E86, MWA_NOP },
  	{ 0x9E87, 0x9E87, ccastles_bankswitch_w },
diff -rC2 new/src/drivers/centiped.c src/src/drivers/centiped.c
*** new/src/drivers/centiped.c	Mon Aug 18 22:16:00 1997
--- src/src/drivers/centiped.c	Fri Sep 19 14:34:00 1997
***************
*** 134,137 ****
--- 134,140 ----
  int centiped_trakball_y(int data);
  
+ void centiped_led_0_w(int offset, int data);
+ void centiped_led_1_w(int offset, int data);
+ 
  extern unsigned char *centiped_charpalette,*centiped_spritepalette;
  void centiped_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom);
***************
*** 165,169 ****
  	{ 0x140c, 0x140f, MWA_RAM, &centiped_spritepalette },
  	{ 0x1800, 0x1800, MWA_NOP },
! 	{ 0x1c00, 0x1c07, MWA_NOP },
  	{ 0x1680, 0x1680, MWA_NOP },
  	{ 0x2000, 0x2000, MWA_NOP },
--- 168,175 ----
  	{ 0x140c, 0x140f, MWA_RAM, &centiped_spritepalette },
  	{ 0x1800, 0x1800, MWA_NOP },
! 	{ 0x1c00, 0x1c02, MWA_NOP },
! 	{ 0x1c03, 0x1c03, centiped_led_0_w },
! 	{ 0x1c04, 0x1c04, centiped_led_1_w },
! 	{ 0x1c05, 0x1c07, MWA_NOP },
  	{ 0x1680, 0x1680, MWA_NOP },
  	{ 0x2000, 0x2000, MWA_NOP },
diff -rC2 new/src/drivers/galaxian.c src/src/drivers/galaxian.c
*** new/src/drivers/galaxian.c	Mon Aug 25 20:51:48 1997
--- src/src/drivers/galaxian.c	Thu Sep 18 19:25:02 1997
***************
*** 116,119 ****
--- 116,122 ----
  	{ 0x7001, 0x7001, interrupt_enable_w },
  	{ 0x7800, 0x7800, mooncrst_sound_freq_w },
+ 	{ 0x6000, 0x6000, led_0_w },
+ 	{ 0x6001, 0x6001, led_1_w },
+ 	{ 0x6002, 0x6002, led_2_w },
  	{ 0x6800, 0x6800, mooncrst_background_w },
  	{ 0x6803, 0x6803, mooncrst_noise_w },
***************
*** 134,137 ****
--- 137,143 ----
  	{ 0x7001, 0x7001, interrupt_enable_w },
  	{ 0x7800, 0x7800, mooncrst_sound_freq_w },
+ 	{ 0x6000, 0x6000, led_0_w },
+ 	{ 0x6001, 0x6001, led_1_w },
+ 	{ 0x6002, 0x6002, led_2_w },
  	{ 0x6800, 0x6800, mooncrst_background_w },
  	{ 0x6803, 0x6803, mooncrst_noise_w },
diff -rC2 new/src/drivers/pacman.c src/src/drivers/pacman.c
*** new/src/drivers/pacman.c	Sat Sep  6 23:10:18 1997
--- src/src/drivers/pacman.c	Fri Sep 19 16:02:08 1997
***************
*** 92,96 ****
  	{ 0x5002, 0x5002, MWA_NOP },
  	{ 0x5003, 0x5003, pengo_flipscreen_w },
! 	{ 0x5004, 0x5007, MWA_NOP },
  	{ 0x5040, 0x505f, pengo_sound_w, &pengo_soundregs },
  	{ 0x5060, 0x506f, MWA_RAM, &spriteram_2 },
--- 92,98 ----
  	{ 0x5002, 0x5002, MWA_NOP },
  	{ 0x5003, 0x5003, pengo_flipscreen_w },
! 	{ 0x5004, 0x5004, led_0_w },
! 	{ 0x5005, 0x5005, led_1_w },
! 	{ 0x5006, 0x5007, MWA_NOP },
  	{ 0x5040, 0x505f, pengo_sound_w, &pengo_soundregs },
  	{ 0x5060, 0x506f, MWA_RAM, &spriteram_2 },
*** new/src/drivers/pacman.c	Sat Sep  6 23:10:18 1997
--- src/src/drivers/pacman.c	Thu Sep 18 17:05:30 1997
diff -rC2 new/src/drivers/rallyx.c src/src/drivers/rallyx.c
*** new/src/drivers/rallyx.c	Sat Sep  6 18:15:30 1997
--- src/src/drivers/rallyx.c	Thu Sep 18 18:07:38 1997
***************
*** 145,149 ****
  	{ 0xa180, 0xa180, rallyx_play_sound_w },
  	{ 0xa181, 0xa181, interrupt_enable_w },
! 	{ 0xa182, 0xa186, MWA_NOP },
  	{ 0x0000, 0x3fff, MWA_ROM },
  	{ 0x8014, 0x801f, MWA_RAM, &spriteram },	/* these are here just to initialize */
--- 145,152 ----
  	{ 0xa180, 0xa180, rallyx_play_sound_w },
  	{ 0xa181, 0xa181, interrupt_enable_w },
! 	{ 0xa182, 0xa183, MWA_NOP },
! 	{ 0xa184, 0xa184, led_0_w },
! 	{ 0xa185, 0xa185, led_1_w },
! 	{ 0xa186, 0xa186, MWA_NOP },
  	{ 0x0000, 0x3fff, MWA_ROM },
  	{ 0x8014, 0x801f, MWA_RAM, &spriteram },	/* these are here just to initialize */
diff -rC2 new/src/drivers/sbasketb.c src/src/drivers/sbasketb.c
*** new/src/drivers/sbasketb.c	Fri Aug 29 21:39:46 1997
--- src/src/drivers/sbasketb.c	Fri Sep 19 14:45:32 1997
***************
*** 58,63 ****
          { 0x3c80, 0x3c80, MWA_NOP },  /* ??? */
          { 0x3c81, 0x3c81, interrupt_enable_w },  /* Interrupt enable */
!         { 0x3c83, 0x3c83, MWA_NOP },  /* Coin light 1 */
!         { 0x3c84, 0x3c84, MWA_NOP },  /* Coin light 2 */
          { 0x3c85, 0x3c85, MWA_NOP },  /* Counter gets incremented on every interrupt */
          { 0x3d00, 0x3d00, sound_command_w },  /* Sound Command */
--- 58,63 ----
          { 0x3c80, 0x3c80, MWA_NOP },  /* ??? */
          { 0x3c81, 0x3c81, interrupt_enable_w },  /* Interrupt enable */
!         { 0x3c83, 0x3c83, MWA_NOP },  /* Coin counter 1 */
!         { 0x3c84, 0x3c84, MWA_NOP },  /* Coin counter 2 */
          { 0x3c85, 0x3c85, MWA_NOP },  /* Counter gets incremented on every interrupt */
          { 0x3d00, 0x3d00, sound_command_w },  /* Sound Command */
diff -rC2 new/src/machine/ccastles.c src/src/machine/ccastles.c
*** new/src/machine/ccastles.c	Sat Aug  9 19:26:44 1997
--- src/src/machine/ccastles.c	Fri Sep 19 15:27:42 1997
***************
*** 92,93 ****
--- 92,97 ----
    }
  }
+ 
+ void ccastles_led_0_w(int offset, int data) {
+   led_w(0, ~data);
+ }
diff -rC2 new/src/machine/centiped.c src/src/machine/centiped.c
*** new/src/machine/centiped.c	Fri Aug  8 19:47:24 1997
--- src/src/machine/centiped.c	Fri Sep 19 14:32:34 1997
***************
*** 89,90 ****
--- 89,99 ----
    return(res);
  }
+ 
+ void centiped_led_0_w(int offset, int data)
+ {
+     led_w(0, ~data>>7);
+ }
+ void centiped_led_1_w(int offset, int data)
+ {
+     led_w(1, ~data>>7);
+ }
diff -rC2 new/src/machine/missile.c src/src/machine/missile.c
*** new/src/machine/missile.c	Fri Aug 29 21:34:58 1997
--- src/src/machine/missile.c	Fri Sep 19 16:07:10 1997
***************
*** 151,154 ****
--- 151,156 ----
  	if(address == 0x4800){
  		ctrld = data & 1;
+ 		led_w(0, ~data>>1);
+ 		led_w(1, ~data>>2);
  		return;
  	}
diff -rC2 new/src/machine/starwars.c src/src/machine/starwars.c
*** new/src/machine/starwars.c	Sun Sep  7 03:09:06 1997
--- src/src/machine/starwars.c	Fri Sep 19 15:53:42 1997
***************
*** 144,147 ****
--- 144,148 ----
     printf("led3 [%d]\n",((data&0x80)>>7) );
     #endif
+    led_w(2, data>>7);
     }
  /**************************************************/
***************
*** 151,154 ****
--- 152,156 ----
     printf("led2 [%d]\n",((data&0x80)>>7) );
     #endif
+    led_w(1, data>>7);
     }
  /**************************************************/
***************
*** 173,176 ****
--- 175,179 ----
     printf("led1 [%d]\n",((data&0x80)>>7) );
     #endif
+    led_w(0, data>>7);
     }
  /**************************************************/
