/***************************************************************************

  pacthink.c

  The strategy of Pac-Man.

  Module added by Sean Gugler on 97-09-10
  modified 97-08-27 Sean Gugler - version 0.8

***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"
#include <random.h>

#include "pacdefs.h"

extern const char * const dTXT[16];
#define random_0to(x) ((x * (random()&0xffff)) >> 16)

int bFruitShown;
int nPenalty = 0;
int nEatenGhost = -1;
int bEatenFruit = 0;

#define DOT(x,y) (x>0 && x<27 && y>0 && y<31) ? (VID(x,y)==0x10 || VID(x,y)==0x12) : 0

/* from pactape.c */

extern int nLevel;
extern int nScoreLevel;
extern int bBeginLevel;
extern const int nSpriteFruit;
extern int nSpritePacman;
extern int xPacman, yPacman;
extern int x,y;
extern int bPoweredUp;

extern long pactape_get_score();

/***************************************************************************

  Update state for ranking purposes

***************************************************************************/
void pacthink_update_state()
{
    if (bBeginLevel)
        nPenalty = 0;
    else
    {
        /* Compute penalty for eating power pellet
           but not all 4 ghosts */
        if (bPoweredUp && (RAM[0x4dd0]>=nEatenGhost)) nEatenGhost = RAM[0x4dd0];
        else if (nEatenGhost>=0)
        {
            if (nEatenGhost<4)
            {
                debug_msg ("PENALTY-GHOSTS.");
                nPenalty += 5000;
            }
            nEatenGhost = RAM[0x4da6] ? RAM[0x4dd0] : -1;
        }

        /* Compute penalty for missing the fruit */
        if (VID(14,18)==0x85 || VID(14,18)==0x8d)
        {
            if (!bEatenFruit)
                debug_msg ("FRUIT-EATEN.");
            bEatenFruit = 1;
            bFruitShown = 1;
        }
        else if ((7 != spriteram_2[nSpriteFruit  ]) ||
                 (8 != spriteram_2[nSpriteFruit+1]) )
            bFruitShown = 1;
        else
        {
            if (bFruitShown)
            {
                if (!bEatenFruit)
                {
                    debug_msg ("PENALTY-FRUIT.");
                    nPenalty += 5000;
                }
                else
                    bEatenFruit = 0;
                bFruitShown = 0;
            }
        }
    }
}

/***************************************************************************

  Rank the current game status

***************************************************************************/
long pacthink_score()
{
    return 40000 + pactape_get_score() - nScoreLevel - nPenalty;
}

/***************************************************************************

  Choose a direction

***************************************************************************/
int pacthink_choose (int dirs)
{
    int dir = 0;
    if (dirs)
    {
#if 0 /* RANDOM */
        do dir = 1 << (random()&3);
        while (!(dirs & dir));
#else
        int rank[16];
        int r = -1;
        int i;
        rank[dD] = rank[dR] = rank[dL] = rank[dU] = 0;

        if (bFruitShown && !bEatenFruit)
        {
            rank[dR] = 50 + 14-x;
            rank[dL] = 50 + x-14;
            rank[dD] = 50 + 18-y;
            rank[dU] = 50 + y-18;
        }
        else if (bPoweredUp)
        {
            int xPac = x*8-4;
            int yPac = y*8+12;
            int xGhost, yGhost;
            int dx, dy;
            int rx=300;
            int ry=300;
            int nSwap = RAM[0x4da4];
            for (i=4; i<11; i+=2)
            {
                int nGhost = i/2;
                if (nGhost == 5) nGhost = 1;
                if (nGhost == nSwap) nGhost = 2;
                else if (nGhost == 2 && nSwap) nGhost = nSwap;

                /* Skip if not edible */
                if (RAM[0x4da6+nGhost] && RAM[0x4d9f+nGhost]==1 && !RAM[0x4dab+nGhost])
                {
                    xGhost = 239 - spriteram_2[i];
                    yGhost = 272 - spriteram_2[i+1];
                    dx = (xGhost-xPac);
                    dy = (yGhost-yPac);

#if 0
                    sprintf (debugText[0], "%d.%d.", dx, dy);
                    debug_msg (debugText[0]);
#endif
                    if (abs(dx) + abs(dy) < abs(rx) + abs(ry))
                    {
                        rx = dx;
                        ry = dy;
                    }
                }
            }
            if (rx==300) rx=ry=0;
            rank[dR] = 300 + rx;
            rank[dL] = 300 - rx;
            rank[dD] = 300 + ry;
            rank[dU] = 300 - ry;
        }
        else
        {
#if 0
            int rows[32];
            int cols[28];
            memset(rows,0,32*sizeof(int));
            memset(cols,0,28*sizeof(int));
            for (i=0x3bf; i>=0x040; i--)
            {
                if (videoram[i]==0x10 || videoram[i]==0x12)
                {
                    rows[i&0x1f]++;
                    cols[29-(i>>5)]++;
                }
            }
            /* Deliberately reversed; prefer sparse dot areas */
            for (i=27; i>=0; i--)
            {
                     if (x<i) rank[dL]+=cols[i];
                else if (x>i) rank[dR]+=cols[i];
            }
            for (i=31; i>=0; i--)
            {
                     if (y<i) rank[dU]+=rows[i];
                else if (y>i) rank[dD]+=rows[i];
            }
            /* However, don't prefer NO-dot areas */
            if (rank[dL] == 0)
            {
                rank[dL] = rank[dR];
                rank[dR] = 0;
            }
            else if (rank[dR] == 0)
            {
                rank[dR] = rank[dL];
                rank[dL] = 0;
            }
            if (rank[dU] == 0)
            {
                rank[dU] = rank[dD];
                rank[dD] = 0;
            }
            else if (rank[dD] == 0)
            {
                rank[dD] = rank[dU];
                rank[dU] = 0;
            }
            if (VID(x,y-1)==0x10)
                rank[dU]+=10;
            if (VID(x,y+1)==0x10)
                rank[dD]+=10;
            if (VID(x-1,y)==0x10)
                rank[dL]+=10;
            if (VID(x+1,y)==0x10)
                rank[dR]+=10;
#endif
            int rad = 1;
            while (rank[dD] == 0 &&
                   rank[dR] == 0 &&
                   rank[dL] == 0 &&
                   rank[dU] == 0 )
            {
                if (DOT(x,y+rad)) rank[dD]+=rad;
                if (DOT(x+rad,y)) rank[dR]+=rad;
                if (DOT(x-rad,y)) rank[dL]+=rad;
                if (DOT(x,y-rad)) rank[dU]+=rad;

                for (i=rad-1; i ; i--)
                {
                    if (DOT(x+rad,y+i  )) {rank[dR]+=rad; rank[dD]+=i  ;};
                    if (DOT(x-rad,y+i  )) {rank[dL]+=rad; rank[dD]+=i  ;};
                    if (DOT(x+rad,y-i  )) {rank[dR]+=rad; rank[dU]+=i  ;};
                    if (DOT(x-rad,y-i  )) {rank[dL]+=rad; rank[dU]+=i  ;};
                    if (DOT(x+i  ,y+rad)) {rank[dR]+=i  ; rank[dD]+=rad;};
                    if (DOT(x-i  ,y+rad)) {rank[dL]+=i  ; rank[dD]+=rad;};
                    if (DOT(x+i  ,y-rad)) {rank[dR]+=i  ; rank[dU]+=rad;};
                    if (DOT(x-i  ,y-rad)) {rank[dL]+=i  ; rank[dU]+=rad;};
                }
                if (DOT(x+rad,y+rad)) {rank[dR]+=rad;rank[dD]+=rad;};
                if (DOT(x-rad,y+rad)) {rank[dL]+=rad;rank[dD]+=rad;};
                if (DOT(x+rad,y-rad)) {rank[dR]+=rad;rank[dU]+=rad;};
                if (DOT(x-rad,y-rad)) {rank[dL]+=rad;rank[dU]+=rad;};

#if 0
                sprintf (debugText[0], "%d:-%d-%d-%d-%d.", rad, rank[dD], rank[dR], rank[dL], rank[dU]);
                debug_msg (debugText[0]);
#endif
                rad++;
            }
        }
#if 0
        sprintf (debugText[0], "%s:-%d-%d-%d-%d.", dTXT[dirs], rank[dD], rank[dR], rank[dL], rank[dU]);
        debug_msg (debugText[0]);
#endif
        for (i=dU; i<=dD; i<<=1)
        {
            if ((dirs & i) &&
                ((rank[i] > r) || (rank[i]==r && random()&1)))
            {
                dir = i;
                r = rank[i];
            }
        }
#endif
    }
    return dir;
}
